#pragma once
#include "vector.h"

class Quaternion {
public:
    Scalar re;
    Vector im;

    Quaternion(Scalar _re, Vector _im): re(_re), im(_im) {}
    Quaternion(Scalar a, Scalar b, Scalar c, Scalar d): re(d), im(a,b,c) {}
    Quaternion(Scalar a): re(a), im() {}

    void conjugate() { im = -im; }
    Quaternion conjugated() const { return Quaternion(re, -im); }
    Quaternion operator~() const { return conjugated(); }
};

inline Quaternion operator*(const Quaternion& a, const Quaternion& b) {
    return Quaternion(
        a.re * b.re - dot(a.im, b.im),
        a.re * b.im + a.im * b.re + cross(a.im, b.im)
    );
}

inline Vector rotate(Quaternion q, Vector v) { return (q * Quaternion(0, v) * ~q).im; }
